// Generated by CoffeeScript 2.5.1
(function() {
  var _, file, fs, http, notFound, options, path, serverError, stat, timeout, util;

  _ = require('lodash');

  fs = require('fs');

  http = require('http');

  path = require('path');

  util = require('util');

  stat = util.promisify(fs.stat);

  file = function(params, resolve, reject, stream) {
    if (!params.file) {
      return reject({
        handler: 'serverError'
      });
    }
    return Promise.resolve().then(function() {
      return stat(params.file);
    }).then((stats) => {
      var code, headers, range;
      range = this._range(params.headers['range'], stats.size);
      code = range ? 206 : 200;
      headers = {
        'Accept-Ranges': 'bytes'
      };
      if (range) {
        headers['Content-Range'] = `bytes ${range.start}-${range.end}/${stats.size}`;
        headers['Content-Length'] = 1 + range.end - range.start;
      } else {
        headers['Content-Length'] = stats.size;
      }
      return {code, headers, range};
    }).then(function({code, headers, range}) {
      if (params.method === 'HEAD') {
        return {code, headers};
      }
      return new Promise(function(resolve, reject) {
        var readStream;
        return readStream = fs.createReadStream(params.file, range).on('open', function() {
          return stream.start(code, headers);
        }).on('error', function(error) {
          reject({error});
          return readStream.close();
        }).on('data', function(chunk) {
          return stream.write(chunk);
        }).on('end', function() {
          stream.end({code, headers});
          resolve({code, headers});
          return readStream.close();
        });
      });
    }).then(function(result) {
      return resolve(result);
    }).catch(function(error) {
      if (error.code === 'ENOENT') {
        return reject({
          handler: 'notFound'
        });
      } else {
        return reject({error});
      }
    });
  };

  notFound = function(params, resolve) {
    var body, code, filePath, headers, std;
    code = params.code || 404;
    headers = {
      'Content-Type': 'text/html'
    };
    std = function() {
      return `<h1>${code} ${http.STATUS_CODES[code]}</h1>
<pre>${params.method} ${params.uri}</pre>`;
    };
    if (!this.options.notFound) {
      body = std();
      return resolve({code, headers, body});
    } else {
      filePath = path.resolve(process.cwd(), this.options.notFound);
      return fs.readFile(filePath, function(error, data) {
        body = error || !data ? std() : data;
        return resolve({code, headers, body});
      });
    }
  };

  serverError = function(params, resolve) {
    var body, code, headers, text;
    text = `${params.method} ${params.uri}`;
    if (params.error) {
      text += `\nMessage: ${params.error.message}`;
    }
    code = params.code || 500;
    headers = {
      'Content-Type': 'text/html'
    };
    body = `<h1>${code} ${http.STATUS_CODES[code]}</h1>
<pre>${text}</pre>`;
    return resolve({code, headers, body});
  };

  timeout = function(params, resolve) {
    var body, code, headers;
    code = params.code || 504;
    headers = {
      'Content-Type': 'text/html'
    };
    body = `<h1>${code} ${http.STATUS_CODES[code]}</h1>
<pre>${params.method} ${params.uri}</pre>`;
    return resolve({code, headers, body});
  };

  options = function(params, resolve) {
    return resolve({
      code: 204
    });
  };

  module.exports = {file, notFound, serverError, timeout, options};

}).call(this);
