// Generated by CoffeeScript 2.5.1
(function() {
  var Server, _, defaultHandlers, fs, http, https, mime, minimatch, path, pkg, url;

  _ = require('lodash');

  fs = require('fs');

  pkg = require('../package.json');

  url = require('url');

  path = require('path');

  mime = require('mime');

  http = require('http');

  https = require('https');

  minimatch = require('minimatch');

  defaultHandlers = require('./handlers');

  Server = (function() {
    var server;

    class Server {
      defaults() {
        return {
          port: 8000,
          host: '0.0.0.0',
          logs: false,
          index: 'index.html',
          https: false,
          key: null,
          cert: null,
          cors: false,
          timeout: 30000
        };
      }

      routes() {
        return [];
      }

      handlers() {
        return {};
      }

      constructor(userOptions = {}, userRoutes, userHandlers, exitCallback) {
        var handlers, options, routes;
        options = {};
        handlers = {};
        routes = [];
        Object.defineProperty(this, 'module', {
          enumerable: true,
          get: () => {
            return this.constructor.module;
          }
        });
        Object.defineProperty(this, 'version', {
          enumerable: true,
          get: () => {
            return this.constructor.version;
          }
        });
        Object.defineProperty(this, 'exitCallback', {
          get: function() {
            return exitCallback;
          }
        });
        Object.defineProperty(this, 'options', {
          enumerable: true,
          get: function() {
            return options;
          },
          set: (newOptions) => {
            options = _.assign(options, _.result(this, 'defaults'), newOptions);
            return this.restart();
          }
        });
        Object.defineProperty(this, '_routes', {
          get: function() {
            return _.concat(routes, {
              pattern: '**',
              handler: 'file'
            });
          },
          set: function(newRoutes) {
            if (_.isObject(newRoutes)) {
              newRoutes = _.map(newRoutes, function(val, key) {
                return {
                  pattern: key,
                  handler: val
                };
              });
            }
            newRoutes = _.chain(newRoutes).filter('pattern').filter('handler').value();
            return routes = _.unionBy(routes, newRoutes, 'pattern');
          }
        });
        Object.defineProperty(this, '_handlers', {
          get: function() {
            return handlers;
          },
          set: function(newHandlers) {
            return handlers = _.assign(handlers, defaultHandlers, newHandlers);
          }
        });
        Object.defineProperty(this, '_server', {
          get: function() {
            return server;
          }
        });
        this._routes = _.result(this, 'routes');
        this._handlers = _.result(this, 'handlers');
        this._routes = userRoutes;
        this._handlers = userHandlers;
        this.options = userOptions;
      }

      start() {
        if (this.options.https) {
          server = this._startHTTP();
        } else {
          server = this._startHTTP();
        }
        return this;
      }

      restart() {
        var ref;
        this._stopLogs();
        this._startLogs();
        if ((ref = this._server) != null) {
          ref.close();
        }
        this.start();
        return this;
      }

      stop(callback) {
        var ref, ref1;
        if ((ref = this._server) != null) {
          ref.close();
        }
        if ((ref1 = this._loger) != null) {
          ref1.end();
        }
        if (typeof this.exitCallback === "function") {
          this.exitCallback();
        }
        if (typeof callback === "function") {
          callback();
        }
        return this;
      }

      log(string) {
        var ref;
        if ((ref = this._logger) != null) {
          ref.write(string + '\n');
        }
        if (typeof this._log === "function") {
          this._log(string);
        }
        return this;
      }

      request(req, res) {
        var req_handler, req_params, time;
        time = new Date();
        req_params = null;
        req_handler = null;
        return Promise.resolve().then(() => {
          return this._requestData(req);
        }).then((data) => {
          return this._navigate(req, data);
        }).then((params) => {
          req_params = params;
          req_handler = params.handler;
          return this._handle(params.handler, params, req, res);
        }).catch(function(params) {
          if (params.handler || params.body || params.streamed) {
            return params;
          }
          throw params;
        }).then((params) => {
          var error_params;
          if (params.handler) {
            error_params = _.assign({}, req_params, params);
            return this._handle(params.handler, error_params, req, res);
          }
          return params;
        }).then((params) => {
          return this.response(res, params, req_params);
        }).catch((error) => {
          var error_params;
          this.log(`[${time.toJSON()}] Error: ${error.message || 'none'}`);
          error_params = _.assign({}, req_params, error);
          return this._handle('serverError', error_params, req, res).then((result) => {
            return this.response(res, result, req_params);
          });
        }).then((params) => {
          var host, log;
          host = path.join(req.headers.host || 'localhost:' + this.options.port);
          log = `[${time.toJSON()}]`;
          log += ` (+${Date.now() - time}ms):`;
          log += ` ${params.code}`;
          log += `\t${host}`;
          log += ` ${req_params.method}`;
          log += ` ${req_params.uri}`;
          if (req_params.file) {
            log += `\t${req_params.file}`;
          }
          if (req.headers['user-agent']) {
            log += `\t(${req.headers['user-agent']})`;
          }
          return this.log(log);
        });
      }

      response(res, params, req_params) {
        if (params.streamed) {
          res.end();
        } else {
          res.writeHead(params.code || (params.body ? 200 : 204), this._responseHeaders(params, req_params));
          res.write(params.body || '');
          res.end();
        }
        return params;
      }

      _startHTTP() {
        return http.createServer((req, res) => {
          return this.request(req, res);
        }).listen(this.options.port, this.options.host);
      }

      _startHTTPS() {
        var cert;
        cert = this._getCert();
        return https.createServer(cert, (req, res) => {
          return this.request(req, res);
        }).listen(this.options.port, this.options.host);
      }

      _getCert() {
        var cert, error, key;
        if (_.isEmpty(this.options.key)) {
          throw new Error("Path to key file demanded for running HTTPS server: --key=/path/to/server.key");
        }
        if (_.isEmpty(this.options.cert)) {
          throw new Error("Path to certificate file demanded for running HTTPS server: --key=/path/to/server.cert");
        }
        try {
          key = fs.readFileSync(path.resolve(this.options.key));
        } catch (error1) {
          error = error1;
          throw new Error(`Can't open key file ${this.options.key} (error.code)`);
        }
        try {
          cert = fs.readFileSync(path.resolve(this.options.cert));
        } catch (error1) {
          error = error1;
          throw new Error(`Can't open cert file ${this.options.cert} (error.code)`);
        }
        return {key, cert};
      }

      _stopLogs() {
        var ref;
        if ((ref = this._logger) != null) {
          ref.end();
        }
        delete this._logger;
        return delete this._log;
      }

      _startLogs() {
        if (!this.options.logs) {
          return;
        }
        if (typeof this.options.logs === 'string') {
          return this._logger = fs.createWriteStream(this.options.logs, {
            flags: 'a'
          });
        } else {
          return this._log = console.log;
        }
      }

      _requestData(req) {
        return new Promise(function(resolve, reject) {
          var data;
          data = [];
          req.on('data', function(chunk) {
            return data.push(chunk);
          });
          req.on('end', function() {
            return resolve(Buffer.concat(data));
          });
          return req.on('error', function(error) {
            return reject(error);
          });
        });
      }

      _responseHeaders(response, request) {
        var allowHeaders, allowMethod, headers, ref, ref1, ref2, requestHeaders, requestMethods, srvName;
        srvName = `${this.module}/${this.version}`;
        headers = {
          'Server': srvName,
          'X-Server': srvName
        };
        if (request.file) {
          headers['Content-Type'] = mime.getType(request.file);
        }
        if (this.options.cors) {
          allowHeaders = ((ref = request.headers) != null ? (ref1 = ref['access-control-request-headers']) != null ? ref1.split(/,\s*/) : void 0 : void 0) || [];
          requestHeaders = _.keys(request.headers || {});
          requestHeaders = _.union(requestHeaders, allowHeaders);
          allowMethod = (ref2 = request.headers) != null ? ref2['access-control-request-method'] : void 0;
          requestMethods = [request.method || 'GET'];
          requestMethods = _.compact(_.uniq([request.method, allowMethod]));
          headers['Access-Control-Allow-Origin'] = this.options.cors === true ? '*' : this.options.cors;
          headers['Access-Control-Allow-Headers'] = requestHeaders.join(',');
          headers['Access-Control-Allow-Methods'] = requestMethods.join(',');
        }
        return _.defaults(response.headers, headers);
      }

      _range(range = '', size = 0) {
        var end, firstRangeStr, start;
        if (String(range).indexOf('bytes=') !== 0) {
          return null;
        }
        firstRangeStr = range.replace('bytes=', '').split(',')[0];
        if (!(firstRangeStr.indexOf('-') > -1)) {
          return null;
        }
        [start, end] = firstRangeStr.split('-');
        start = parseInt(start, 10);
        end = parseInt(end, 10);
        if (_.isNaN(start)) {
          start = size - end;
          end = size - 1;
        } else if (_.isNaN(end)) {
          end = size - 1;
        }
        if (end > size - 1) {
          end = size - 1;
        }
        if (_.isNaN(start) || _.isNaN(end) || start > end || start < 0) {
          return null;
        }
        return {start, end};
      }

      _cookie(cookie) {
        var cookieArr, cookieObj, el, elSplit, i, len;
        if (!cookie) {
          return {};
        }
        cookieArr = cookie.split(';');
        cookieObj = {};
        for (i = 0, len = cookieArr.length; i < len; i++) {
          el = cookieArr[i];
          elSplit = el.split('=');
          if (elSplit.length >= 2) {
            cookieObj[elSplit[0].trim()] = elSplit.slice(1).join('=').trim();
          }
        }
        return cookieObj;
      }

      _navigate(req, data) {
        var client_ip, cookie, file, hash, headers, method, params, pathname, query, range, ref, root, search;
        method = req.method;
        headers = req.headers;
        client_ip = headers['x-forwarded-for'] || ((ref = req.connection) != null ? ref.remoteAddress : void 0);
        cookie = this._cookie(headers['cookie']);
        range = this._range(headers['range']);
        params = {
          uri: req.url,
          method,
          headers,
          client_ip,
          cookie,
          range,
          data
        };
        try {
          ({pathname, query, search, hash} = url.parse(req.url, true));
        } catch (error1) {
          return _.assign(params, {
            handler: 'notFound',
            query: {},
            search: null,
            hash: null
          });
        }
        params = _.assign(params, {pathname, query, search, hash});
        if (method === 'OPTIONS') {
          return _.assign(params, {
            handler: 'options'
          });
        }
        root = path.resolve(process.cwd(), this.options.root || './');
        file = pathname;
        file = file.replace(/\/$/, `/${this.options.index}`);
        file = file.replace(/^\//, "");
        file = path.join(root, file);
        params.file = file;
        params.handler = this._getHandler(pathname) || 'notFound';
        return params;
      }

      _getHandler(pathname) {
        var found;
        found = _.find(this._routes, ({pattern, handler}) => {
          return minimatch(pathname, pattern, {
            matchBase: true
          }) && (_.isFunction(handler) || _.isFunction(this._handlers[handler]));
        });
        return (found != null ? found.handler : void 0) || '';
      }

      _handle(handler, params, req, res) {
        return new Promise((resolve, reject) => {
          var end, error, start, stream, write;
          start = (code, headers) => {
            return res.writeHead(code, this._responseHeaders({headers}, params));
          };
          write = function(chunk) {
            return res.write(chunk);
          };
          end = function(res_params) {
            return resolve(_.assign(res_params, {
              streamed: true
            }));
          };
          stream = {start, write, end};
          setTimeout(function() {
            return reject(_.assign(params, {
              handler: 'timeout'
            }));
          }, this.options.timeout);
          try {
            if (_.isFunction(handler)) {
              return handler.call(this, params, resolve, reject, stream);
            } else {
              return this._handlers[handler].call(this, params, resolve, reject, stream);
            }
          } catch (error1) {
            error = error1;
            return reject(_.assign(params, {
              handler: 'serverError',
              error: error
            }));
          }
        });
      }

    };

    server = null;

    Server.module = pkg.name;

    Server.version = pkg.version;

    return Server;

  }).call(this);

  module.exports = Server;

}).call(this);
